package geniusweb.protocol.session;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.Value;
import geniusweb.protocol.SessionResult;
import geniusweb.references.Parameters;
import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithParameters;
import geniusweb.references.PartyWithProfile;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;

public class SessionResultTest extends GeneralTests<SessionResult> {
	private final ObjectMapper jackson = new ObjectMapper();
	private final RuntimeException error = new RuntimeException("test");

	private final String ISSUE1 = "issue1";
	private SessionResult result1, result1a, result2, result3;// , result4;
	private String errorstring; // created dynamically: eclipse and maven
								// generate different stacktrace.
	private String jsonstring = "{\"participants\":[{\"party\":{\"partyref\":\"party1\",\"parameters\":{}},\"profile\":\"profile1\"},{\"party\":{\"partyref\":\"party2\",\"parameters\":{}},\"profile\":\"profile2\"}],\"agreement\":{\"issuevalues\":{\"issue1\":\"a\"}},\"error\":null}";

	@Before
	public void before() throws URISyntaxException, JsonProcessingException {
		errorstring = "\"error\":{\"java.lang.RuntimeException\":"
				+ jackson.writeValueAsString(error) + "}";
		System.out.println(errorstring);

		PartyWithParameters party1 = new PartyWithParameters(
				new PartyRef("party1"), new Parameters());
		PartyWithParameters party2 = new PartyWithParameters(
				new PartyRef("party2"), new Parameters());

		PartyWithProfile partyprofile1 = new PartyWithProfile(party1,
				new ProfileRef("profile1"));
		PartyWithProfile partyprofile2 = new PartyWithProfile(party2,
				new ProfileRef("profile2"));

		Map<String, Value> issuevalues1 = new HashMap<>();
		issuevalues1.put(ISSUE1, new DiscreteValue("a"));
		Bid bid1 = new Bid(issuevalues1);

		// different order but that shouldn't matter
		Map<String, Value> issuevalues2 = new HashMap<>();
		issuevalues2.put(ISSUE1, new DiscreteValue("b"));
		Bid bid2 = new Bid(issuevalues2);

		result1 = new SessionResult(Arrays.asList(partyprofile1, partyprofile2),
				bid1, null);
		result1a = new SessionResult(
				Arrays.asList(partyprofile1, partyprofile2), bid1, null);
		result2 = new SessionResult(Arrays.asList(partyprofile1, partyprofile2),
				bid2, null);
		result3 = new SessionResult(Arrays.asList(partyprofile2, partyprofile1),
				bid2, null);
		// IGNORE ERROR for now, it fails somewhere deep in maven suddenly.
		// result4 = new SessionResult(Arrays.asList(partyprofile1,
		// partyprofile2), bid1, error);
	}

	@Override
	public List<List<SessionResult>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(result1, result1a),
				Arrays.asList(result2), Arrays.asList(result3)
		// ,Arrays.asList(result4)
		);
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"SessionResult.*party1.*profile1.*,.*party2.*profile2.*Bid.*issue1=\"a\".*null.",
				"SessionResult.*party1.*profile1.*,.*party2.*profile2.*Bid.*issue1=\"b\".*null.*",
				"SessionResult.*party2.*profile2.*,.*party1.*profile1.*Bid.*issue1=\"b\".*null.*"
		// ,"SessionResult.*party1.*profile1.*,.*party2.*profile2.*Bid.*issue1=\"a\".*null.*"

		);
	}

	@Test
	public void serializeTest() throws JsonProcessingException {
		System.out.println(jackson.writeValueAsString(result1));
		assertEquals(jsonstring, jackson.writeValueAsString(result1));
	}

	@Test
	public void deserializeTest() throws IOException {
		SessionResult act = jackson.readValue(jsonstring, SessionResult.class);
		assertEquals(result1, act);
	}

}
